﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using NPersistence.NHibernate.annotationSupport.mapping;
using NHibernate.Cfg;
using System.IO;
using NHibernate;
using NHibernate.Dialect;
using NHibernate.Util;
using NPersistence.NHibernate.Event;
using NHibernate.Event;

namespace NPersistence.NHibernate.annotationSupport
{
    public class AnnotaionConfig
    {
        private static AnnotaionConfig INSTANCE = null;
        private Dictionary<string, string> properties;
        private Dialect dialect = null;

        private AnnotaionConfig()
        { }

        public static AnnotaionConfig getInstance(Dictionary<string, string> props)
        {
            if (INSTANCE == null)
            {
                INSTANCE = new AnnotaionConfig();
            }
            if (props != null)
            {
                INSTANCE.properties = props;
            }
            return INSTANCE;
        }

        internal Dialect getDialect()
        {
            if (dialect != null)
            {
                return dialect;
            }
            try
            {
                dialect = Dialect.GetDialect(properties);
                return dialect;
            }
            catch (HibernateException e)
            {
                throw new PersistenceException("The property 'dialect' is missing from the configuration. Please add it to the persistence.xml file");
            }
        }
        public XmlSerializer getNpaToHbmConverter()
        {
            return new XmlSerializer(typeof(HbmMapping));
        }

        internal void configure(List<string> classList, Configuration configuration)
        {
            if (classList == null || classList.Count == 0)
            {
                return;
            }
            string outputFolder = null;
            properties.TryGetValue(AvailableSettings.TEMP_FILES_OUTPUT_FOLDER, out outputFolder);
            
            XmlSerializer xmlSerializer = getNpaToHbmConverter();
            // XMLs may change each other during creation.
            // list them up before sending all to NHIbernate.
            IDictionary<Type, HbmMapping> mappings = new Dictionary<Type, HbmMapping>();
            foreach (string str in classList)
            {
                Type typ = ReflectHelper.ClassForFullName(str);

                if (typ == null)
                {
                    throw new PersistenceException("The class <" + str + "> mentioned in the persistence.xml file was not found in the classpath");
                }
                // This addition is a workaround to a bug!!!
                // Hibernate doesn't know how to get the entities out of queries without this.
                configuration.Imports[typ.Name] = typ.FullName + ", " + typ.Assembly;

                if (!tryHbmAddition(configuration, typ))
                {
                    HbmMapping mapping = new HbmMapping(typ);
                    mappings[typ] = mapping;
                }
            }
            foreach (Type mappedType in mappings.Keys)
            {
                HbmMapping mapping = mappings[mappedType];
                MemoryStream memoryStream = new MemoryStream();
                xmlSerializer.Serialize(memoryStream, mapping);
                if (outputFolder != null)
                {
                    try
                    {
                        TextWriter WriteFileStream = new StreamWriter(outputFolder + Path.DirectorySeparatorChar + mappedType.Name + ".hbm.xml");
                        xmlSerializer.Serialize(WriteFileStream, mapping);
                        WriteFileStream.Dispose();
                        WriteFileStream.Close();
                    }
                    catch (Exception e)
                    { }
                }
                memoryStream.Seek(0, SeekOrigin.Begin);
                configuration.AddInputStream(memoryStream);

                memoryStream.Dispose();
                memoryStream.Close();
            }
            if (PrePersistCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PreInsertEventListeners = addItem<IPreInsertEventListener>(configuration.EventListeners.PreInsertEventListeners, PrePersistCallbackHandler.getInstance());
            }
            if (PostPersistCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PostInsertEventListeners = addItem<IPostInsertEventListener>(configuration.EventListeners.PostInsertEventListeners, PostPersistCallbackHandler.getInstance());
            }
            if (PostLoadCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PostLoadEventListeners = addItem<IPostLoadEventListener>(configuration.EventListeners.PostLoadEventListeners, PostLoadCallbackHandler.getInstance());
            }
            if (PreUpdateCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PreUpdateEventListeners = addItem<IPreUpdateEventListener>(configuration.EventListeners.PreUpdateEventListeners, PreUpdateCallbackHandler.getInstance());
            }
            if (PostUpdateCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PostUpdateEventListeners = addItem<IPostUpdateEventListener>(configuration.EventListeners.PostUpdateEventListeners, PostUpdateCallbackHandler.getInstance());
            }
            if (PreRemoveCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PreDeleteEventListeners = addItem<IPreDeleteEventListener>(configuration.EventListeners.PreDeleteEventListeners, PreRemoveCallbackHandler.getInstance());
            }
            if (PostRemoveCallbackHandler.getInstance().hasRegistrations())
            {
                configuration.EventListeners.PostDeleteEventListeners = addItem<IPostDeleteEventListener>(configuration.EventListeners.PostDeleteEventListeners, PostRemoveCallbackHandler.getInstance());
            }
            // destroy
            MappingCache.destroy();
            INSTANCE = null;
        }

        private T[] addItem<T>(T[] array, T addedItem)
        {
            if (array != null)
            {
                int size = array.Length;
                Array.Resize(ref array, size + 1);
                array[size] = addedItem;
                return array;
            }
            return new T[] { addedItem };
        }
       
        private bool tryHbmAddition(Configuration configuration, Type typ)
        {
            Assembly assembly = typ.Assembly;
            string hbmPath = typ.Name + ".hbm.xml";
            System.IO.Stream stream = null;
            try
            {
                string[] resources = assembly.GetManifestResourceNames();
                foreach (string resourceName in resources)
                {
                    if (resourceName.EndsWith(hbmPath))
                    {
                        stream = assembly.GetManifestResourceStream(resourceName);
                        configuration.AddInputStream(stream);
                        return true;
                    }
                }
            }
            catch (FileNotFoundException e)
            {
                return false;
            }
            catch (MappingException e)
            {
                return false;
            }
            finally
            {
                if (stream != null)
                {
                    stream.Close();
                }
            }
            return false;
        }
    }
}