﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    class CascadeTypeMapping
    {
        /* Mappings:        
        ALL - all, 
        PERSIST - persist, 
        MERGE - merge, 
        REMOVE - delete,
        REFRESH - refresh,
        DETACH - evict
        OneToMany.orphanRemoval - delete-orphan
        */
        // these are the cascade types in HBM:
        // 1)persist, merge, delete, save-update, evict, 
        // replicate, lock, refresh; 
        // 2) delete-orphan;
        // 3) all 

        public static String mapCascadeType(CascadeType[] cascadeTypeArr, Object relationship)
        {
            StringBuilder sb = new StringBuilder();
            bool commaNeeded = false;
            if (relationship is OneToMany && ((OneToMany)relationship).OrphanRemoval)
            {
                sb.Append("delete-orphan");
                commaNeeded = true;
            }
            if (cascadeTypeArr != null)
            {
                foreach (CascadeType cascadeType in cascadeTypeArr)
                {
                    if (cascadeType == null)
                    {
                        continue;
                    }
                    if (commaNeeded)
                    {
                        sb.Append(",");
                    }
                    if (cascadeType == CascadeType.ALL)
                    {
                        sb.Append("all");
                    }
                    else if (cascadeType == CascadeType.PERSIST)
                    {
                        sb.Append("persist");
                    }
                    else if (cascadeType == CascadeType.MERGE)
                    {
                        sb.Append("merge");
                    }
                    else if (cascadeType == CascadeType.REMOVE)
                    {
                        sb.Append("delete");
                    }
                    else if (cascadeType == CascadeType.REFRESH)
                    {
                        sb.Append("refresh");
                    }
                    else if (cascadeType == CascadeType.DETACH)
                    {
                        sb.Append("evict");
                    }
                    commaNeeded = true;
                }
            }
            
            String ret = sb.ToString();
            if (ret.Length == 0)
            {
                return null;
            }
            return ret;
        }
    }
}
