﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NPersistence.NHibernate.annotationSupport.mapping;
using NHibernate.UserTypes;
using NPersistence.Ext;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    class ClassUtils
    {

        internal static void mapProperties(Type entity, PropertyHelper embeddingProperty, List<object> itemsList, List<PropertyHelper> idList, HbmClass clss, bool includeInheritedProperties)
        {
            foreach (PropertyHelper propertyInfo in MappingUtils.getProperties(entity, embeddingProperty, includeInheritedProperties))
            {
                Transient transient = propertyInfo.getSingleAttribute<Transient>();
                if (transient != null)
                {
                    continue;
                }
                object[] attributes = propertyInfo.GetCustomAttributes(false);
                bool mainAttrFound = false;
                foreach (object attr in attributes)
                {
                    if (attr is Id || attr is EmbeddedId)
                    {
                        if (idList == null)
                        {
                            throw new PersistenceException("Id or EmbeddedId should only be used in Entity classes not Embedable classes.");
                        }
                        mainAttrFound = true;
                        idList.Add(propertyInfo);
                        break;
                    }
                    else if (attr is Basic)
                    {
                        mainAttrFound = true;
                        handleProperty(propertyInfo, itemsList, clss);
                        break;
                    }
                    else if (attr is Embedded)
                    {
                        mainAttrFound = true;
                        HbmComponent comp = new HbmComponent(propertyInfo);
                        itemsList.Add(comp);
                        break;
                    }
                    else if (attr is OneToOne)
                    {
                        mainAttrFound = true;
                        addOneToOne(propertyInfo, itemsList);
                        break;
                    }
                    else if (attr is ManyToOne)
                    {
                        mainAttrFound = true;
                        HbmManyToOne prop = new HbmManyToOne(propertyInfo);
                        itemsList.Add(prop);
                        break;
                    }
                    else if (attr is ElementCollection || attr is OneToMany || attr is ManyToMany)
                    {
                        mainAttrFound = true;
                        addCollection(propertyInfo, itemsList);
                        break;
                    }
                    else if (attr is Version)
                    {
                        mainAttrFound = true;
                        handleProperty(propertyInfo, itemsList, clss);
                        break;
                    }
                    else if (attr is Access)
                    {
                        if (((Access)attr).Value == AccessType.FIELD)
                        {
                            throw new PersistenceException("Field access type is not supported");
                        }
                    }
                }
                if (!mainAttrFound)
                {
                    mainAttrFound = true;
                    handleProperty(propertyInfo, itemsList, clss);
                }
            }
        }

        private static void handleProperty(PropertyHelper propertyInfo, List<object> itemsList, HbmClass clss)
        {
            Version version = propertyInfo.getSingleAttribute<Version>();
            if (version != null)
            {
                if (clss == null)
                {
                    throw new PersistenceException("Version should only be declared on Entities not Embeddebles");
                }
                HbmVersion prop = new HbmVersion(propertyInfo);
                clss.Item1 = prop;
            }
            else
            {
                HbmProperty prop = new HbmProperty(propertyInfo);
                itemsList.Add(prop);
            }
        }

        private static void addOneToOne(PropertyHelper propertyInfo, List<object> itemsList)
        {
            JoinTableHelper joinTbl = MappingUtils.getJoinTable(propertyInfo, null, null);
            JoinColumn[] joinCol = MappingUtils.getJoinColumn(propertyInfo);
            OneToOne oneToOne = propertyInfo.getSingleAttribute<OneToOne>();
            object prop = null;

            if (String.IsNullOrEmpty(oneToOne.MappedBy) || joinTbl != null || (joinCol != null && joinCol.Length > 0))
            {
                prop = new HbmManyToOne(propertyInfo);
            }
            else
            {
                prop = new HbmOneToOne(propertyInfo);
            }
            itemsList.Add(prop);
        }

        private static void addCollection(PropertyHelper propertyInfo, List<object> itemsList)
        {
            Type propertyType = propertyInfo.PropertyType;

            if (propertyType.IsInterface && addCollectionMapping(propertyType, propertyInfo, itemsList))
            {
                return;
            }
            Type[] interfaces = propertyType.GetInterfaces();

            foreach (Type intrfac in interfaces)
            {
                if (addCollectionMapping(intrfac, propertyInfo, itemsList))
                {
                    return;
                }
            }
            throw new PersistenceException("Unsupported type of collection: " + propertyType.FullName + ". The following types are expected"
                + "Iesi.Collections.ISet, Iesi.Collections.Generic.ISet, IUserCollectionType, System.Collections.IList, System.Collections.Generic.IList, System.Collections.Generic.ICollection"
                + "System.Collections.ICollection, System.Collections.IDictionary, System.Collections.Generic.IDictionary");
        }

        private static bool addCollectionMapping(Type intrfac, PropertyHelper propertyInfo, List<object> itemsList)
        {
            String interfaceName = intrfac.FullName;
            if (interfaceName.Contains("Iesi.Collections.ISet") || interfaceName.Contains("Iesi.Collections.Generic.ISet"))
            {
                itemsList.Add(new HbmSet(propertyInfo));
                return true;
            }
            else if (intrfac.IsAssignableFrom(typeof(IUserCollectionType)) || interfaceName.Contains("System.Collections.IList") || interfaceName.Contains("System.Collections.Generic.IList"))
            {
                OrderColumn orderCol = propertyInfo.getSingleAttribute<OrderColumn>();

                if (orderCol != null)
                {
                    itemsList.Add(new HbmList(propertyInfo));
                    return true;
                }
                else
                {
                    itemsList.Add(new HbmBag(propertyInfo));
                    return true;
                }
            }
            else if (interfaceName.Contains("System.Collections.Generic.ICollection") || interfaceName.Contains("System.Collections.ICollection"))
            {
                itemsList.Add(new HbmBag(propertyInfo));
                return true;
            }
            else if (interfaceName.Contains("System.Collections.IDictionary") || interfaceName.Contains("System.Collections.Generic.IDictionary"))
            {
                itemsList.Add(new HbmMap(propertyInfo));
                return true;
            }
            return false;
        }

        internal static void applyEntityAttributes(Type entity)
        {
            //apply attirubte override for mappedSuperclass
            applyAttributeOverride((AttributeOverride[])entity.GetCustomAttributes(typeof(AttributeOverride), false), null);
            //apply attribute overrides for my properties
            applyAttributeOverride(entity, null);
            //apply unique and indices
            applyEntityIndex(entity);
            applyEntityUnique(entity);
        }

        private static void applyAttributeOverride(Type type, PropertyHelper embeddingProperty)
        {
            foreach (PropertyHelper propertyInfo in MappingUtils.getProperties(type, embeddingProperty))
            {
                if (propertyInfo.isEmbedded())
                {
                    Type embeddedType = RelationUtils.getRelationTargetEntity(null, propertyInfo);
                    //First, recursivly apply overrieds
                    applyAttributeOverride(embeddedType, propertyInfo);
                    if (MappingUtils.isDictionary(propertyInfo.PropertyType))
                    {
                        String keyName;
                        Type keyType = MappingUtils.findMapKeyClass(propertyInfo, embeddedType, out keyName);
                        if (MappingUtils.isEmbeddable(keyType))
                        {
                            applyAttributeOverride(keyType, propertyInfo);
                        }
                    }
                    // Apply override to current property
                    applyAttributeOverride(propertyInfo.getAttributeArr<AttributeOverride>(), propertyInfo.getPath());
                }
            }
        }

        private static void applyAttributeOverride(AttributeOverride[] attributeOverrides, String path)
        {
            foreach (AttributeOverride attributeOverride in attributeOverrides)
            {
                if (String.IsNullOrEmpty(attributeOverride.PropertyName))
                {
                    throw new PersistenceException("The PropertyName field in the AttributeOverride annotation is mandatory");
                }
                if (String.IsNullOrEmpty(attributeOverride.Name))
                {
                    throw new PersistenceException("The Name field (represents the column name) in the AttributeOverride annotation is mandatory");
                }
                String completePath = String.IsNullOrEmpty(path) ? attributeOverride.PropertyName : path + "." + attributeOverride.PropertyName;
                HbmColumn hbmCol = MappingCache.getInstance().getColumn(completePath);
                if (hbmCol == null)
                {
                    throw new PersistenceException("The PropertyName field in the AttributeOverride annotation was not found <" + attributeOverride.PropertyName + ">.");
                }
                hbmCol.apply((Column)attributeOverride);
            }
        }

        private static void applyEntityIndex(Type entity)
        {
            Index[] indicesArr = (Index[])entity.GetCustomAttributes(typeof(Index), false);
            foreach (Index ix in indicesArr)
            {
                if(ix.Members != null)
                {
                    if (string.IsNullOrEmpty(ix.Name))
                    {
                        ix.Name = MappingUtils.getNextIndexCounter() + "_ix";
                    }
                    foreach(String member in ix.Members)
                    {
                        HbmColumn hbmCol = MappingCache.getInstance().getColumn(member);
                        if (hbmCol == null)
                        {
                            throw new PersistenceException("The member <"+member+">in the Index annotation in entity <"+entity.Name+"> was not found.");
                        }
                        hbmCol.index = String.IsNullOrEmpty(hbmCol.index) ? ix.Name : hbmCol.index + "," + ix.Name;
                    }
                }
                
            }
 
        }

        private static void applyEntityUnique(Type entity)
        {
            Unique[] uniqueArr = (Unique[])entity.GetCustomAttributes(typeof(Unique), false);
            foreach (Unique uq in uniqueArr)
            {
                if (uq.Members != null)
                {
                    if (string.IsNullOrEmpty(uq.Name))
                    {
                        uq.Name = MappingUtils.getNextIndexCounter() + "_uq";
                    }
                    foreach (String member in uq.Members)
                    {
                        HbmColumn hbmCol = MappingCache.getInstance().getColumn(member);
                        if (hbmCol == null)
                        {
                            throw new PersistenceException("The member <" + member + ">in the Unique annotation in entity <" + entity.Name + "> was not found.");
                        }
                        hbmCol.uniquekey = String.IsNullOrEmpty(hbmCol.uniquekey) ? uq.Name : hbmCol.uniquekey + "," + uq.Name;
                    }
                }

            }

        }
    }
}
