using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using NPersistence.NHibernate.src.annotationSupport.mapping;
using System.Reflection;
using NHibernate.UserTypes;
using NPersistence.NHibernate.Event;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	partial class HbmClass 
	{
        List<object> itemsList = new List<object>();
        
        Dictionary<String, object> itemsMap = new Dictionary<String, object>();
        Type idCalssType = null;
        public HbmClass(Type entity, bool includeInheritedProperties)
            : this()
        {
            //Handle class level Meta data
            this.name = MappingUtils.getStringClassDef(entity);
            if (entity.IsAbstract)
            {
                this.@abstract = true;
                this.abstractSpecified = true;
            }
            object[] attributeArr = entity.GetCustomAttributes(false);

            foreach (object attr in attributeArr)
            {
                if (attr is Table)
                {
                    setTable((Table)attr);
                }
                else if (attr is IdClass)
                {
                    idCalssType = ((IdClass)attr).Value;
                }
                else if (attr is Access)
                {
                    if (((Access)attr).Value == AccessType.FIELD)
                    {
                        throw new PersistenceException("Field access type is not supported");
                    }
                }
            }
           
            //Handle property level meta data
            List<PropertyHelper> idList = new List<PropertyHelper>();
            ClassUtils.mapProperties(entity, null, itemsList, idList, this, includeInheritedProperties);
            handleId(idList);
            ClassUtils.applyEntityAttributes(entity);
            SecondaryTable[] secondaryTblArr = (SecondaryTable[])entity.GetCustomAttributes(typeof(SecondaryTable), false);
            foreach (SecondaryTable secondaryTbl in secondaryTblArr)
            {
                HbmJoin hbmJoin = new HbmJoin(entity, secondaryTbl, itemsList);
                addJoin(hbmJoin);
                Dictionary<String, String> uniqueConstraints = MappingUtils.getUniqueConstraints(entity, secondaryTbl.Name);
                MappingUtils.applyUniqueConstraints(uniqueConstraints, new Object[] { hbmJoin });
            }
            if (!String.IsNullOrEmpty(this.table))
            {
                Dictionary<String,String> uniqueConstraints = MappingUtils.getUniqueConstraints(entity, this.table);
                MappingUtils.applyUniqueConstraints(uniqueConstraints, this.Items);
            }
            this.Items = itemsList.ToArray();
        }

        private void addJoin(HbmJoin hbmJoin)
        {
            if (this.Items1 != null)
            {
                int len = this.Items1.Length + 1;
                Array.Resize(ref this.Items1, len);
                this.Items1[len - 1] = hbmJoin;
            }
            else
            {
                this.Items1 = new Object[] { hbmJoin };
            }
        }

        private void handleId(List<PropertyHelper> idList)
        {
            if (idList.Count == 0)
            {
                throw new ArgumentException("Id or EmbeddedId is expected on entity <" +this.name + ">.");
            }
            
            if (idCalssType != null)
            {
                this.Item = new HbmCompositeId(idCalssType, idList);
            }
            else 
            {
                if (idList.Count > 1)
                {
                    throw new ArgumentException("Only 1 Id or EmbeddedId are expected if IdClass is not present");
                }
                PropertyHelper propInfo = idList[0];
                object[] attributes = propInfo.GetCustomAttributes(false);
                
                foreach (object attr in attributes)
                {
                    if (attr is Id )
                    {
                        this.Item = new HbmId(propInfo);
                        break;
                    }
                    else if (attr is EmbeddedId)
                    {
                        this.Item = new HbmCompositeId(propInfo);
                        break;
                    }
                }
            }
            
        }

        void setTable(Table tbl)
        {
            this.table = tbl.Name;
            if (tbl.Schema != null && tbl.Schema.Length > 0)
            {
                this.schema = tbl.Schema;
            }

            if (tbl.Catalog != null && tbl.Catalog.Length > 0)
            {
                this.catalog = tbl.Catalog;
            }
        }

        private static bool findaa(object bk)
        { return false; }

		public HbmCompositeId CompositeId
		{
			get { return Item as HbmCompositeId; }
		}

		public HbmVersion Version
		{
			get { return Item1 as HbmVersion; }
		}

		public HbmTimestamp Timestamp
		{
			get { return Item1 as HbmTimestamp; }
		}

		[XmlIgnore]
		public IEnumerable<HbmJoin> Joins
		{
			get { return Items1 != null ? Items1.OfType<HbmJoin>(): new HbmJoin[0]; }
		}

		[XmlIgnore]
		public IEnumerable<HbmSubclass> Subclasses
		{
			get { return Items1 != null ? Items1.OfType<HbmSubclass>() : new HbmSubclass[0]; }
		}

		[XmlIgnore]
		public IEnumerable<HbmJoinedSubclass> JoinedSubclasses
		{
			get { return Items1 != null ? Items1.OfType<HbmJoinedSubclass>() : new HbmJoinedSubclass[0]; }
		}

		[XmlIgnore]
		public IEnumerable<HbmUnionSubclass> UnionSubclasses
		{
			get { return Items1 != null ? Items1.OfType<HbmUnionSubclass>() : new HbmUnionSubclass[0]; }
		}

		#region Implementation of IEntityMapping

		protected HbmMeta[] Metadatas
		{
			get { return meta ?? new HbmMeta[0]; }
		}

		public string EntityName
		{
			get { return entityname; }
		}

		public string Name
		{
			get { return name; }
		}

		public string Node
		{
			get { return node; }
		}

		public string Proxy
		{
			get { return proxy; }
		}

		public bool? UseLazy
		{
			get { return lazySpecified ? lazy : (bool?)null; }
		}

		public HbmTuplizer[] Tuplizers
		{
			get { return tuplizer ?? new HbmTuplizer[0]; }
		}

		public bool DynamicUpdate
		{
			get { return dynamicupdate; }
		}

		public bool DynamicInsert
		{
			get { return dynamicinsert; }
		}

		public int? BatchSize
		{
			get { return batchsizeSpecified ? batchsize : (int?) null; }
		}

		public bool SelectBeforeUpdate
		{
			get { return selectbeforeupdate; }
		}

		public string Persister
		{
			get { return persister; }
		}

		public bool? IsAbstract
		{
			get { return abstractSpecified ? @abstract : (bool?) null; }
		}

		public HbmSynchronize[] Synchronize
		{
			get { return synchronize ?? new HbmSynchronize[0]; }
		}

		#endregion

		#region Implementation of IEntityDiscriminableMapping

		public string DiscriminatorValue
		{
			get { return discriminatorvalue; }
		}

		#endregion

		#region Implementation of IEntitySqlsMapping

		public HbmLoader SqlLoader
		{
			get { return loader; }
		}

		public HbmCustomSQL SqlInsert
		{
			get { return sqlinsert; }
		}

		public HbmCustomSQL SqlUpdate
		{
			get { return sqlupdate; }
		}

		public HbmCustomSQL SqlDelete
		{
			get { return sqldelete; }
		}

		#endregion

		
	}
}