using System;
using System.Collections.Generic;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmJoin 
	{

        public HbmJoin(Type entity)
            : this()
        {
            handleTable(entity);
            handleKey(entity, true);

            List<object> itemsList = new List<object>();
            ClassUtils.mapProperties(entity, null, itemsList, null, null, false);
            this.Items = itemsList.ToArray();
            ClassUtils.applyEntityAttributes(entity);    
        }

        public HbmJoin(Type entity, SecondaryTable secondaryTbl, List<object> itemsList)
        {
            this.table = secondaryTbl.Name;
            this.schema = MappingUtils.getNotEmptyString(secondaryTbl.Schema);
            this.catalog = MappingUtils.getNotEmptyString(secondaryTbl.Catalog);
            
            handleKey(entity, false);
            List<object> ownItems = new List<object>();
            foreach (Object obj in itemsList)
            {
                if (obj is TableResident)
                {
                    if (this.table.Equals(((TableResident)obj).getTable()))
                    {
                        ownItems.Add(obj);
                    }
                }
            }

            foreach(Object obj in ownItems)
            {
                itemsList.Remove(obj);
            }
            this.Items = ownItems.ToArray();

        }

        private void handleKey(Type entity, bool isInheritence)
        {
            PrimaryKeyJoinColumn[] keyArr = (PrimaryKeyJoinColumn[])entity.GetCustomAttributes(typeof(PrimaryKeyJoinColumn), false);
            if (keyArr.Length > 0)
            {
                Type keyEntity = isInheritence ? entity.BaseType : entity;
                this.key = new HbmKey(keyArr, keyEntity);
            }
            else
            {
                this.key = new HbmKey();
            }
        }

        private void handleTable(Type entity)
        {
            Object[] tableArr = entity.GetCustomAttributes(typeof(Table), false);
            if (tableArr.Length > 0)
            {
                Table tbl = (Table)tableArr[0];
                this.table = tbl.Name;
                this.schema = MappingUtils.getNotEmptyString(tbl.Schema);
                this.catalog = MappingUtils.getNotEmptyString(tbl.Catalog);
            }
            else
            {
                this.table = entity.Name + "_" + "JOIN";
            }
        }

		#region Implementation of IEntitySqlsMapping

		public HbmLoader SqlLoader
		{
			get { return null; }
		}

		public HbmCustomSQL SqlInsert
		{
			get { return sqlinsert; }
		}

		public HbmCustomSQL SqlUpdate
		{
			get { return sqlupdate; }
		}

		public HbmCustomSQL SqlDelete
		{
			get { return sqldelete; }
		}

		#endregion

		#region Implementation of IPropertiesContainerMapping

        /*
		[XmlIgnore]
		public IEnumerable<IEntityPropertyMapping> Properties
		{
			get { return Items != null ? Items.Cast<IEntityPropertyMapping>() : new IEntityPropertyMapping[0]; }
		}
        */
		#endregion
	}
}