using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using System.Reflection;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmManyToMany 
	{
        [System.Xml.Serialization.XmlIgnoreAttribute()]
        public Type targetType = null;
        [System.Xml.Serialization.XmlIgnoreAttribute()]
        private bool isInverse;
        public HbmManyToMany(PropertyHelper propertyInfo, PropertyHelper inverseProperty)
            : this()
        {
            isInverse = inverseProperty != null;

            ManyToMany manyToMany = propertyInfo.getSingleAttribute<ManyToMany>();
            targetType = RelationUtils.getRelationTargetEntity(manyToMany, propertyInfo);
            this.@class = MappingUtils.getStringClassDef(targetType);
            JoinTableHelper joinTable = MappingUtils.getJoinTable(propertyInfo, propertyInfo.ReflectedType, targetType);
            if (joinTable != null)
            {
                if (isInverse)
                {
                    throw new ArgumentException("Inverse side of ManyToMany relation should not have joinTable definition");
                }
            }
            if (isInverse)
            {
                joinTable = MappingUtils.getJoinTable(inverseProperty, inverseProperty.ReflectedType, propertyInfo.ReflectedType);
            }
            if (joinTable != null)
            {
                handleJoinTable(joinTable);
            }
            else if(isInverse)
            {
                JoinColumn[] attrArr = MappingUtils.getJoinColumn(inverseProperty);
                if (attrArr != null && attrArr.Length > 0)
                {
                    for (int i = 0; i < attrArr.Length; i++)
                    {
                        attrArr[i] = MappingUtils.setJoinColumnDefaults(attrArr[i], inverseProperty, i);
                    }
                    this.Items = MappingUtils.getJoinColumns(attrArr, targetType, ref this.propertyref);
                }
            }
        }

        private void handleJoinTable(JoinTableHelper joinTable)
        {
            if (isInverse)
            {
                this.Items = MappingUtils.getJoinColumns(joinTable.joinColumns, targetType, ref this.propertyref);
            }
            else
            {
                this.Items = MappingUtils.getJoinColumns(joinTable.inverseJoinColumns, targetType, ref this.propertyref);
            }
        }

        

		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get { return Items != null ? Items.OfType<HbmColumn>() : AsColumns(); }
		}

		#endregion

		private IEnumerable<HbmColumn> AsColumns()
		{
			if (string.IsNullOrEmpty(column))
			{
				yield break;
			}
			else
			{
				yield return new HbmColumn
				{
					name = column,
					unique = unique,
					uniqueSpecified = true
				};
			}
		}

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get { return Items != null ? Items.OfType<HbmFormula>() : AsFormulas(); }
		}

		private IEnumerable<HbmFormula> AsFormulas()
		{
			if (string.IsNullOrEmpty(formula))
			{
				yield break;
			}
			else
			{
				yield return new HbmFormula { Text = new[] { formula } };
			}
		}

		#endregion

		#region Implementation of IRelationship

		public string EntityName
		{
			get { return entityname; }
		}

		public string Class
		{
			get { return @class; }
		}

		public HbmNotFoundMode NotFoundMode
		{
			get { return notfound; }
		}

		#endregion
	}
}