using System.Collections.Generic;
using System.Xml.Serialization;
using System.Reflection;
using System;
using NPersistence.NHibernate.src.annotationSupport.mapping;
using log4net;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    public partial class HbmMap : RelationObject
    {
        private ILog logger = LogManager.GetLogger("NPersistence.NHibernate");
        private String mappedBy = null;
        private Type targetType = null;
        private bool isInexed = false;
        
        public HbmMap(PropertyHelper propertyInfo)
            : this()
        {
            handleMapKey(propertyInfo);
            RelationUtils.handleRelation(ref this.name, propertyInfo, ref this.Item1, ref this.lazy, ref this.lazySpecified, ref this.table, ref this.schema, ref this.catalog, ref this.key, ref targetType, ref this.cascade, ref this.inverse, isInexed, ref this.orderby, ref mappedBy);
            MappingCache.getInstance().registerRelation(propertyInfo.getPath(), propertyInfo.ReflectedType, this);
        }
        
        public void handleMapKey(PropertyHelper propertyInfo)
        {
            targetType = RelationUtils.getRelationTargetEntity(null, propertyInfo);
            String mapKeyName;
            Type mapKeyClass = MappingUtils.findMapKeyClass(propertyInfo, this.targetType, out mapKeyName);
            // if mapKeyName is null, the key of the map is not a property on the mapped object.
            if (mapKeyName == null)
            {
                // NHibernate does not support bidirectional map when the map key is not one of the
                // value's property. in this case, remove 'inverse'.
                logger.Warn("NHibernate does not support bidirectional map when the map key is not one of the value's properties. In this case the user must take care of updates on both sides of the relation. <" + this.name + ">.");
                isInexed = true;
                this.inverse = false;
            }

            if (MappingUtils.isEntity(mapKeyClass))
            {
                this.Item = new HbmMapKeyManyToMany(propertyInfo, mapKeyClass);
            }
            else if (MappingUtils.isEmbeddable(mapKeyClass))
            {
                this.Item = new HbmCompositeMapKey(propertyInfo, mapKeyClass);
            }
            else // simple type
            {   
                this.Item = new HbmMapKey(propertyInfo, mapKeyClass, this.targetType, mapKeyName);
            }
        }

        public bool isIndexed()
        {
            return isInexed;
        }
        public String getMappedBy()
        {
            return mappedBy;
        }

        public Type getTargetType()
        {
            return targetType;
        }

        #region Implementation of IEntityPropertyMapping

        public string Name
        {
            get { return name; }
        }

        public string Access
        {
            get { return access; }
        }

        public bool IsLazyProperty
        {
            get { return false; }
        }

        public bool OptimisticLock
        {
            get { return optimisticlock; }
        }

        #endregion

        #region Implementation of IReferencePropertyMapping

        public string Cascade
        {
            get { return cascade; }
        }

        #endregion

        #region Overrides of AbstractDecoratable

        protected HbmMeta[] Metadatas
        {
            get { return meta ?? new HbmMeta[0]; }
        }

        #endregion

        #region Implementation of ICollectionSqlsMapping

        public HbmLoader SqlLoader
        {
            get { return loader; }
        }

        public HbmCustomSQL SqlInsert
        {
            get { return sqlinsert; }
        }

        public HbmCustomSQL SqlUpdate
        {
            get { return sqlupdate; }
        }

        public HbmCustomSQL SqlDelete
        {
            get { return sqldelete; }
        }

        public HbmCustomSQL SqlDeleteAll
        {
            get { return sqldeleteall; }
        }

        #endregion

        #region Implementation of ICollectionPropertiesMapping

        public bool Inverse
        {
            get { return inverse; }
        }

        public bool Mutable
        {
            get { return mutable; }
        }

        public string OrderBy
        {
            get { return orderby; }
        }

        public string Where
        {
            get { return where; }
        }

        public int? BatchSize
        {
            get { return batchsizeSpecified ? batchsize : (int?)null; }
        }

        public string PersisterQualifiedName
        {
            get { return persister; }
        }

        public string CollectionType
        {
            get { return collectiontype; }
        }

        public HbmCollectionFetchMode? FetchMode
        {
            get { return fetchSpecified ? fetch : (HbmCollectionFetchMode?)null; }
        }

        public HbmOuterJoinStrategy? OuterJoin
        {
            get { return outerjoinSpecified ? outerjoin : (HbmOuterJoinStrategy?)null; }
        }

        public HbmCollectionLazy? Lazy
        {
            get { return lazySpecified ? lazy : (HbmCollectionLazy?)null; }
        }

        public string Table
        {
            get { return table; }
        }

        public string Schema
        {
            get { return schema; }
        }

        public string Catalog
        {
            get { return catalog; }
        }

        public string Check
        {
            get { return check; }
        }

        public object ElementRelationship
        {
            get { return Item1; }
        }

        public string Sort
        {
            get { return sort; }
        }

        public bool? Generic
        {
            get { return genericSpecified ? generic : (bool?)null; }
        }

        [XmlIgnore]
        public IEnumerable<HbmFilter> Filters
        {
            get { return filter ?? new HbmFilter[0]; }
        }

        public HbmKey Key
        {
            get { return key; }
        }

        public HbmCache Cache
        {
            get { return cache; }
        }

        #endregion
    }
}