using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using System;
using System.Reflection;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmNestedCompositeElement
	{
        public HbmNestedCompositeElement(PropertyHelper propInfo) : base()
        {
            Type targetType = propInfo.PropertyType;
            this.name = propInfo.Name;
            this.@class = MappingUtils.getStringClassDef(targetType);
            List<object> itemsList = new List<object>();
            foreach (PropertyHelper propertyInfo in MappingUtils.getProperties(targetType, propInfo))
            {
                Type type = propertyInfo.PropertyType;
                if (type.IsArray
                    || MappingUtils.isCollection(type)
                    || propertyInfo.getSingleAttribute<Transient>() != null
                    || propertyInfo.getSingleAttribute<OneToOne>() != null)
                {
                    continue;
                }
                if (propertyInfo.getSingleAttribute<ManyToOne>() != null)
                {
                    itemsList.Add(new HbmManyToOne(propertyInfo));
                }
                else if (MappingUtils.isEmbeddable(type) && propertyInfo.getSingleAttribute<Embedded>() != null)
                {
                    itemsList.Add(new HbmNestedCompositeElement(propertyInfo));
                }
                else
                {
                    itemsList.Add(new HbmProperty(propertyInfo));
                }
            }

            this.Items = itemsList.ToArray();
        }
		

		#region Implementation of IComponentMapping

		public string Class
		{
			get { return @class; }
		}

		public HbmParent Parent
		{
			get { return parent; }
		}

		public string EmbeddedNode
		{
			get { return node; }
		}

		public string Name
		{
			get { return name; }
		}

		public string Access
		{
			get { return access; }
		}

		public bool OptimisticLock
		{
			get { return true; }
		}

		#endregion

		#region Overrides of AbstractDecoratable

		protected HbmMeta[] Metadatas
		{
			get { return new HbmMeta[0];}
		}

		#endregion

		public bool IsLazyProperty
		{
			get { return false; }
		}
	}
}