using System.Collections.Generic;
using System.Xml.Serialization;
using System.Reflection;
using System;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmOneToOne 
	{
        public HbmOneToOne(PropertyHelper propertyInfo): this()
        {
            this.name = propertyInfo.Name;
            this.@class = MappingUtils.getStringClassDef(propertyInfo.PropertyType);

            List<Object> itemList = new List<object>();

            OneToOne oneTo1 = propertyInfo.getSingleAttribute<OneToOne>();
            this.cascade = CascadeTypeMapping.mapCascadeType(oneTo1.Cascade, oneTo1);
            if (oneTo1.Fetch == FetchType.EAGER)
            {
                this.lazy = HbmLaziness.False;
            }
            if (oneTo1.TargetEntity != null && !oneTo1.TargetEntity.Equals(typeof(void)))
            {
                this.@class = MappingUtils.getStringClassDef(oneTo1.TargetEntity);
            }

            if (oneTo1.MappedBy != null && oneTo1.MappedBy.Length > 0)
            {
                this.propertyref = oneTo1.MappedBy;
            }
            if (!oneTo1.Optional)
            {
                this.constrained = true;
            }

        }

		#region Implementation of IEntityPropertyMapping

		public string Name
		{
			get { return name; }
		}

		public string Access
		{
			get { return access; }
		}

		public bool OptimisticLock
		{
			get { return true; }
		}

		#endregion	
		
		#region Overrides of AbstractDecoratable

		protected HbmMeta[] Metadatas
		{
			get { return meta ?? new HbmMeta[0]; }
		}

		#endregion

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get { return formula ?? AsFormulas(); }
		}

		private IEnumerable<HbmFormula> AsFormulas()
		{
			if (string.IsNullOrEmpty(formula1))
			{
				yield break;
			}
			else
			{
				yield return new HbmFormula { Text = new[] { formula1 } };
			}
		}

		#endregion

		#region Implementation of IRelationship

		public string EntityName
		{
			get { return entityname; }
		}

		public string Class
		{
			get { return @class; }
		}

		public HbmNotFoundMode NotFoundMode
		{
			get { return HbmNotFoundMode.Exception; }
		}

		#endregion

		public HbmLaziness? Lazy
		{
			get { return lazySpecified ? lazy : (HbmLaziness?)null; }
		}

		public bool IsLazyProperty
		{
			get { return Lazy == HbmLaziness.Proxy; }
		}
	}
}