﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    class MappingCache
    {
        private static MappingCache INSTANCE = null;

        private Dictionary<String, HbmColumn> columnRegistry = new Dictionary<string,HbmColumn>();
        private Dictionary<String, RelationObject> relationRegistry = new Dictionary<string, RelationObject>();
        private Dictionary<String, RelationObject> relationMappedByRegistry = new Dictionary<string, RelationObject>();

        internal void registerColumn(String path, HbmColumn column)
        {
            columnRegistry[path] = column;
        }

        internal HbmColumn getColumn(String path)
        {
            HbmColumn column;
            columnRegistry.TryGetValue(path, out column);
            return column;
        }

        internal bool containsColumn(String path)
        {
            return columnRegistry.ContainsKey(path);
        }

        internal void registerRelation(String path, Type entity, RelationObject relation)
        {

            relationRegistry[entity.FullName + "." + path] = relation;
            String mappedBy = relation.getMappedBy();
            if (!String.IsNullOrEmpty(mappedBy))
            {
                relationMappedByRegistry[relation.getTargetType().FullName + "." + mappedBy] = relation;
            }
        }

        internal RelationObject getMappedByRelation(Type entity, String mappedBy)
        {
            RelationObject relation;
            relationMappedByRegistry.TryGetValue(entity.FullName + "." + mappedBy, out relation);
            return relation;
        }

        internal RelationObject getRelation(Type entity, String path)
        {
            RelationObject relation;
            relationRegistry.TryGetValue(entity.FullName + "." + path, out relation);
            return relation;
        }

        private MappingCache() { }

        internal static MappingCache getInstance()
        {
            return INSTANCE;
        }

        internal static void destroy()
        {
            INSTANCE = null;
        }

        internal static void initColumnRegistry()
        {
            if (INSTANCE == null)
            {
                INSTANCE = new MappingCache();
            }
            else
            {
                INSTANCE.columnRegistry = new Dictionary<string, HbmColumn>();
            }
        }
    }
}
