﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using NHibernate.UserTypes;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    class MappingUtils
    {
        private static int indexCounter = 0;
        internal static int getNextIndexCounter()
        {
            indexCounter++;
            return indexCounter;
        }

        /* getPKtype
        * check first for IdClass
        * else check an id or embeddedId annotation
        */
        public static Type getPKType(Type type, out String pkPropertyName)
        {
            if (!isEntity(type))
            {
                throw new InvalidOperationException("only entities can have ID attributes and <" + type.Name+ "> is not.");
            }
            object[] attributes = type.GetCustomAttributes(typeof(IdClass), false);
            if (attributes != null && attributes.Length > 0)
            {
                pkPropertyName = null;
                return ((IdClass)attributes[0]).Value;
            }
            else
            {
                PropertyHelper[] properties = getProperties(type, null);
                foreach (PropertyHelper prop in properties)
                {
                    Object[] attr = prop.getAttributeArr<Id>();
                    if (attr != null && attr.Length > 0)
                    {
                        pkPropertyName = prop.Name;
                        return prop.PropertyType;
                    }
                    else
                    {
                        attr = prop.getAttributeArr<EmbeddedId>();
                        if (attr != null && attr.Length > 0)
                        {
                            pkPropertyName = prop.Name;
                            return prop.PropertyType;
                        }
                    }
                }
            }
            pkPropertyName = null;
            return null;
        }

        public static Type findPropertyType(String propertyName, Type type)
        {
            String[] mappedByTokens = propertyName.Split(new char[] { '.' }, 2);
            PropertyInfo propInfo = type.GetProperty(mappedByTokens[0]);
            if (propInfo == null)
            {
                throw new ArgumentException("Could not find property: " + propertyName);
            }

            if (mappedByTokens.Length == 1)
            {
                return propInfo.PropertyType;
            }
            else
            {
                return findPropertyType(mappedByTokens[1], propInfo.PropertyType);
            }
        }

        public static bool isEntity(Type type)
        {
            object[] attr = type.GetCustomAttributes(typeof(Entity), false);
            if (attr != null && attr.Length > 0)
            {
                return true;
            }
            return false;
        }

        public static bool isEmbeddable(Type type)
        {
            object[] attr = type.GetCustomAttributes(typeof(Embeddable), false);
            if (attr != null && attr.Length > 0)
            {
                return true;
            }
            return false;
        }

        public static String getStringClassDef(Type type)
        {
            if (type == null)
                return null;
            return type.FullName + ", " + type.Assembly.GetName().Name;
        }

        public static bool isCollection(Type type)
        {
            Type[] interfaces = type.GetInterfaces();
            foreach (Type intrfac in interfaces)
            {
                String interfaceName = intrfac.FullName;
                if (interfaceName.Contains("Iesi.Collections.ISet") 
                    || interfaceName.Contains("Iesi.Collections.Generic.ISet")
                    || intrfac.IsAssignableFrom(typeof(IUserCollectionType)) 
                    || interfaceName.Contains("System.Collections.IList") 
                    || interfaceName.Contains("System.Collections.Generic.IList")
                    || interfaceName.Contains("System.Collections.Generic.ICollection") 
                    || interfaceName.Contains("System.Collections.ICollection")
                    || interfaceName.Contains("System.Collections.IDictionary") 
                    || interfaceName.Contains("System.Collections.Generic.IDictionary"))
                {
                    return true;
                }                
            }
            return false;
        }

        public static String getNotEmptyString(String str)
        {
            if(String.IsNullOrEmpty(str))
            {
                return null;
            }
            return str;
        }

        internal static JoinColumn setJoinColumnDefaults(JoinColumn joinCol, PropertyHelper propertyInfo, int index)
        {
            JoinColumn ret;
            if (joinCol == null)
            {
                ret = new JoinColumn();
            }
            else
            {
                ret = joinCol;
            }
            if(String.IsNullOrEmpty(ret.Name))
            {
                ret.Name = propertyInfo.Name + "_" + index;
            }
            return ret;
        }

        internal static AssociationOverrideHelper getAssociationOverride(Type entity, String path)
        {
            AssociationOverride[] assocOverArr = (AssociationOverride[])entity.GetCustomAttributes(typeof(AssociationOverride), false);
            JoinColumn[] joinColArr = (JoinColumn[])entity.GetCustomAttributes(typeof(JoinColumn), false);
            JoinTable[] joinTblArr = (JoinTable[])entity.GetCustomAttributes(typeof(JoinTable), false);
            return getAssociationOverride(assocOverArr, joinColArr, joinTblArr, path);
        }

        internal static AssociationOverrideHelper getAssociationOverride(PropertyHelper propertyInfo, String path)
        {
            AssociationOverride[] assocOverArr = propertyInfo.getAttributeArr<AssociationOverride>();
            JoinColumn[] joinColArr = propertyInfo.getAttributeArr<JoinColumn>();
            JoinTable[] joinTblArr = propertyInfo.getAttributeArr<JoinTable>();
            return getAssociationOverride(assocOverArr, joinColArr, joinTblArr, path);
        }

        private static AssociationOverrideHelper getAssociationOverride(AssociationOverride[] assocOverArr, JoinColumn[] joinColArr, JoinTable[] joinTblArr, String path)
        {

            if (assocOverArr == null || assocOverArr.Length == 0)
            {
                return null;
            }
            
            foreach (AssociationOverride associationOvrr in assocOverArr)
            {
                if (associationOvrr.Name.Equals(path))
                {
                    List<JoinColumn> joinColsList = new List<JoinColumn>();
                    foreach (JoinColumn col in joinColArr)
                    {
                        if (associationOvrr.Name.Equals(col.AttributeRef))
                        {
                            joinColsList.Add(col);
                        }
                    }
                    JoinTableHelper joinTblHelper = null;
                    foreach (JoinTable joinTbl in joinTblArr)
                    {
                        if (associationOvrr.Name.Equals(joinTbl.AssociationOverrideRef))
                        {
                            joinTblHelper = getJoinTable(joinTbl, joinColArr);
                            break;
                        }
                    }
                    return new AssociationOverrideHelper(associationOvrr.Name, joinTblHelper, joinColsList.ToArray());
                }
            }
            return null;
        }

        private static JoinTableHelper getJoinTable(JoinTable joinTbl, JoinColumn[] joinColArr)
        {
            List<JoinColumn> joinColsList = new List<JoinColumn>();
            List<JoinColumn> inverseJoinColsList = new List<JoinColumn>();
            
            foreach (JoinColumn col in joinColArr)
            {
                if (joinTbl.Name.Equals(col.AttributeRef))
                {
                    if (col.Inverse)
                    {
                        inverseJoinColsList.Add(col);
                    }
                    else
                    {
                        joinColsList.Add(col);
                    }
                }
            }
            return new JoinTableHelper(joinTbl, joinColsList.ToArray(), inverseJoinColsList.ToArray());
        }

        internal static JoinTableHelper getJoinTable(PropertyHelper propertyInfo, Type owninigEntity, Type inverseEntity)
        {
            AssociationOverrideHelper assocOverr = null;
            if (propertyInfo.isOnMappedSuperclass())
            {
                assocOverr = getAssociationOverride(propertyInfo.ReflectedType, propertyInfo.Name);
            }
            else if (isEmbeddable(propertyInfo.DeclaringType))
            {
                String path;
                PropertyHelper embeddingProp = propertyInfo.getEmbeddingProperty(out path);
                if(isDictionary(propertyInfo.PropertyType))
                {
                    path = path + "." + "value";
                }
                assocOverr = getAssociationOverride(embeddingProp, path);
            }
            if (assocOverr != null)
            {
                return assocOverr.joinTable;
            }
            
            JoinTable joinTbl = propertyInfo.getSingleAttribute<JoinTable>();
            
            if (joinTbl == null)
            {
                return null;
            }
            if (String.IsNullOrEmpty(joinTbl.Name))
            {
                joinTbl.Name = getJoinTableName(owninigEntity, inverseEntity);
            }
            JoinColumn[] joinCols = propertyInfo.getAttributeArr<JoinColumn>();
            return getJoinTable(joinTbl, joinCols);
        }

        internal static JoinColumn[] getJoinColumn(PropertyHelper propertyInfo)
        {
            if (propertyInfo.isOnMappedSuperclass())
            {
                AssociationOverrideHelper assocOverr = getAssociationOverride(propertyInfo.ReflectedType, propertyInfo.Name);
                if (assocOverr != null )
                {
                    return assocOverr.joinColumns;
                }
            }
            return propertyInfo.getAttributeArr<JoinColumn>();
        }

        internal static CollectionTableHelper getCollectionTable(PropertyHelper propertyInfo)
        {
            CollectionTable collectionTbl = propertyInfo.getSingleAttribute<CollectionTable>();
            if (collectionTbl == null)
            {
                return null;
            }
            if (String.IsNullOrEmpty(collectionTbl.Name))
            {
                Type holdingEntity = propertyInfo.ReflectedType;
                String collectionAttribute = propertyInfo.Name;
                collectionTbl.Name = getEntityTableName(holdingEntity) + "_" + collectionAttribute;
            }
            List<JoinColumn> joinColsList = new List<JoinColumn>();
            JoinColumn[] joinCols = propertyInfo.getAttributeArr<JoinColumn>();
            foreach (JoinColumn col in joinCols)
            {
                if (collectionTbl.Name.Equals(col.AttributeRef))
                {
                    joinColsList.Add(col);   
                }
            }
            return new CollectionTableHelper(collectionTbl, joinColsList.ToArray());
        }

        internal static PropertyHelper findPropertyByColumn(Type entity, string columnName)
        {
            PropertyHelper[] properties = getProperties(entity, null, true);
            columnName = columnName.ToLower();
            foreach (PropertyHelper prop in properties)
            {
                Column col = prop.findColumn(true);
                if (col.Name.ToLower().Equals(columnName))
                {
                    return prop;
                }
                if (isEmbeddable(prop.PropertyType))
                {
                    AttributeOverride[] attributes = prop.getAttributeArr<AttributeOverride>();
                    foreach (AttributeOverride attOverride in attributes)
                    {
                        if (columnName.Equals(attOverride.Name))
                        {
                            return prop;
                        }
                    }

                    PropertyHelper p = findPropertyByColumn(prop.PropertyType, columnName);
                    if (p != null)
                    {
                        return prop;
                    }
                }
            }
            return null;
        }

        internal static PropertyHelper[] getProperties(Type entity, PropertyHelper embeddingProperty)
        {
            return getProperties(entity, embeddingProperty, false);
        }

        internal static PropertyHelper[] getProperties(Type entity, PropertyHelper embeddingProperty, bool includeInheritedProperties)
        {
            PropertyInfo[] properties = entity.GetProperties();

            // if all properties includeing inherited ones are required, return it all
            // else filter out inherited properties, only properties that are declared on the current entity
            // or properties from mappedSuperclass are returned
            if (properties.Length > 0)
            {
                String keyOrValue = "";
                if (embeddingProperty != null && isDictionary(embeddingProperty.PropertyType))
                {
                    Type target = RelationUtils.getRelationTargetEntity(null, embeddingProperty);
                    if (target.Equals(entity))
                    {
                        keyOrValue = "value";
                    }
                    else
                    {
                        keyOrValue = "key";
                    }
                }
                List<PropertyHelper> propsList = new List<PropertyHelper>();
                foreach (PropertyInfo propertyInfo in properties)
                {
                    if (propertyInfo.GetCustomAttributes(typeof(Transient), false).Length > 0)
                    {
                        continue;
                    }
                    if(isEmbeddable(propertyInfo.DeclaringType))
                    {
                        propsList.Add(new PropertyHelper( propertyInfo, embeddingProperty, keyOrValue));
                    }
                    else if (propertyInfo.DeclaringType.Equals(propertyInfo.ReflectedType))
                    {
                        propsList.Add(new PropertyHelper( propertyInfo));
                    }
                    else if (isOnMappedSupperclass(propertyInfo))
                    {
                        propsList.Add(new PropertyHelper( propertyInfo, true));
                    }
                    else if (includeInheritedProperties && isEntity( propertyInfo.DeclaringType))
                    {
                        propsList.Add(new PropertyHelper( propertyInfo));
                    }
                }
                return propsList.ToArray();
            }
            return new PropertyHelper[0];
        }

        private static bool isOnMappedSupperclass(PropertyInfo propertyInfo)
        {
            Type declaringType = propertyInfo.DeclaringType;
            Type reflectedType = propertyInfo.ReflectedType;
            
            Type entitySuperclass = reflectedType;
            do
            {
                entitySuperclass = entitySuperclass.BaseType;
                if (entitySuperclass.GetCustomAttributes(typeof(MappedSuperclass), false).Length == 0)
                {
                    return false;
                }
            }
            while (!declaringType.Equals(entitySuperclass));
            return true;
        }

        internal static String getEntityTableName(Type entity)
        {
            if (entity == null)
            {
                return "";
            }
            if (!isEntity(entity))
            {
                throw new InvalidOperationException("This function should only be called with Entities and <" + entity.Name + "> is not." );
            }
            Object[] tbl = entity.GetCustomAttributes(typeof(Table), false);
            if (tbl != null && tbl.Length > 0)
            {
                return ((Table)tbl[0]).Name;
            }
            return entity.Name;
        }

        internal static string getJoinTableName(Type entity1, Type entity2)
        {
            if (entity1 == null || entity2 == null)
            {
                return "";
            }
            return getEntityTableName(entity1) + "_" + getEntityTableName(entity2);
        }

        internal static InheritanceType getInheritenceType(Type entity, out DiscriminatorColumn discriminatorCol)
        {
            discriminatorCol = null;
            InheritanceType ?ret = null;
            object[] inheritenceArr = entity.GetCustomAttributes(typeof(Inheritance), false);
            if (inheritenceArr.Length > 0)
            {
                ret = ((Inheritance)inheritenceArr[0]).Strategy;
            }
            else if (entity.BaseType == null || entity.BaseType.GetCustomAttributes(typeof(Entity), false).Length == 0)
            {
                ret = InheritanceType.SINGLE_TABLE;
            }

            if (ret.HasValue && ret.Value == InheritanceType.SINGLE_TABLE)
            {
                object[] discriminatorArr = entity.GetCustomAttributes(typeof(DiscriminatorColumn), false);
                if (discriminatorArr.Length > 0)
                {
                    discriminatorCol = (DiscriminatorColumn)discriminatorArr[0];
                }
                else
                {
                    discriminatorCol = new DiscriminatorColumn();
                }
            }

            if (ret.HasValue)
            {
                return ret.Value;
            }
            return getInheritenceType(entity.BaseType, out discriminatorCol);
        }

        internal static IEnumerable<Type> FindSubClassesOf(Type baseType)
        {
            var assembly = baseType.Assembly;
            return assembly.GetTypes().Where(t => baseType.Equals(t.BaseType));
        }

        internal static bool isDictionary(Type type)
        {
            if (type.FullName.Contains("System.Collections.IDictionary") || type.FullName.Contains("System.Collections.Generic.IDictionary"))
            {
                return true;
            }

            Type[] interfaces = type.GetInterfaces();

            foreach (Type intrfac in interfaces)
            {
                if (intrfac.FullName.Contains("System.Collections.IDictionary") || intrfac.FullName.Contains("System.Collections.Generic.IDictionary"))
                {
                    return true;
                }
            }
            return false;
        }
        
        internal static String getDiscriminatorValue(Type entity, DiscriminatorColumn discriminatorCol)
        {
            Object[] discriminatorValArr = entity.GetCustomAttributes(typeof(DiscriminatorValue), false);
            if (discriminatorValArr.Length > 0)
            {
                return ((DiscriminatorValue)discriminatorValArr[0]).Value;
            }
            else if (discriminatorCol != null)
            {
                // if discriminator column is not string, need to generate a value
                if (discriminatorCol.DiscriminatorType == DiscriminatorType.INTEGER)
                {
                    return entity.Name.GetHashCode().ToString();
                }
                else if (discriminatorCol.DiscriminatorType == DiscriminatorType.CHAR)
                {
                    byte[] byteArr = new byte[1];
                    new Random(0).NextBytes(byteArr);
                    return byteArr[0] + "";
                }
            }
            return entity.Name;
        }


        /**
         * Return a mapping from colName to unique constraint names separated with comma.
         * example:
         * co1-> uq1,uq2
         */
        internal static Dictionary<String,String> getUniqueConstraints(Object entityOrProp, String tableName)
        {
            if (string.IsNullOrEmpty(tableName))
            {
                return null;
            }
            UniqueConstraint[] constraints = null;
            if (entityOrProp is Type)
            {
                constraints = (UniqueConstraint[])((Type)entityOrProp).GetCustomAttributes(typeof(UniqueConstraint), false);
            }
            else if (entityOrProp is PropertyHelper)
            {
                constraints = ((PropertyHelper)entityOrProp).getAttributeArr<UniqueConstraint>();
            }
            if (constraints != null)
            {
                Dictionary<String, String> retDict = new Dictionary<string, string>();
                foreach (UniqueConstraint constraint in constraints)
                {
                    if (tableName.Equals(constraint.TableRef))
                    {
                        String constraintName = string.IsNullOrEmpty(constraint.Name) ? tableName + "uq" + getNextIndexCounter() : constraint.Name;
                        foreach(string col in constraint.ColumnNames)
                        {
                            String colConstraints = null;
                            if(retDict.TryGetValue(col, out colConstraints))
                            {
                                colConstraints = colConstraints + "," + constraintName;
                            }
                            else
                            {
                                colConstraints = constraintName;
                            }
                            retDict[col]=colConstraints;
                        }
                    }
                }
                return retDict;
            }
            return null;
        }

        internal static void applyUniqueConstraints(Dictionary<string, string> uniqueConstraintMap, object[] itemsArr)
        {
            if(uniqueConstraintMap == null || uniqueConstraintMap.Count == 0 || itemsArr == null || itemsArr.Length == 0)
            {
                return;
            }
            foreach (Object hbmObject in itemsArr)
            {
                if(hbmObject is HbmColumn)
                {
                    HbmColumn col = (HbmColumn)hbmObject;
                    String constraints;
                    if(uniqueConstraintMap.TryGetValue(col.name, out constraints))
                    {
                        col.uniquekey = String.IsNullOrEmpty(col.uniquekey)? constraints : col.uniquekey +"," + constraints;
                    }
                }
                else if(hbmObject is HbmProperty)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmProperty)hbmObject).Items);
                }
                else if(hbmObject is HbmManyToOne)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmManyToOne)hbmObject).Items);
                }
                else if(hbmObject is HbmAny)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmAny)hbmObject).column);
                }
                else if(hbmObject is HbmElement)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmElement)hbmObject).Items);
                }
                else if (hbmObject is HbmKey)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmKey)hbmObject).column);
                }
                else if(hbmObject is HbmMapKey)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmMapKey)hbmObject).Items);
                }
                else if(hbmObject is HbmMapKeyManyToMany)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmMapKeyManyToMany)hbmObject).Items);
                }
                else if(hbmObject is HbmKeyManyToOne)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmKeyManyToOne)hbmObject).column);
                }
                else if(hbmObject is HbmKeyProperty)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmKeyProperty)hbmObject).column);
                }
                else if(hbmObject is HbmComponent)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmComponent)hbmObject).Items);
                }
                else if(hbmObject is HbmCompositeElement)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmCompositeElement)hbmObject).Items);
                }
                else if (hbmObject is HbmCompositeId)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmCompositeId)hbmObject).Items);
                }
                else if(hbmObject is HbmNestedCompositeElement)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmNestedCompositeElement)hbmObject).Items);
                }
                else if(hbmObject is HbmArray)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[]{((HbmArray)hbmObject).Item1});
                }
                else if(hbmObject is HbmBag)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[]{((HbmBag)hbmObject).Item});
                }
                else if(hbmObject is HbmDynamicComponent)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmDynamicComponent)hbmObject).Items);
                }
                else if(hbmObject is HbmIdbag)
                {
                    applyUniqueConstraints(uniqueConstraintMap,  new Object[]{((HbmIdbag)hbmObject).Item});
                }
                else if(hbmObject is HbmList)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[]{((HbmList)hbmObject).Item1});
                }
                else if(hbmObject is HbmSet)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[]{((HbmSet)hbmObject).Item});
                }
                else if(hbmObject is HbmPrimitiveArray)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[]{((HbmPrimitiveArray)hbmObject).element});
                }
                else if(hbmObject is HbmMap)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[] { ((HbmMap)hbmObject).Item });
                    applyUniqueConstraints(uniqueConstraintMap, new Object[] { ((HbmMap)hbmObject).Item1 });
                }
                else if(hbmObject is HbmCompositeMapKey)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmCompositeMapKey)hbmObject).Items);
                }

                else if (hbmObject is HbmPrimitiveArray)
                {
                    applyUniqueConstraints(uniqueConstraintMap, new Object[] { ((HbmPrimitiveArray)hbmObject).element });
                }
                else if (hbmObject is HbmSubclass)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmSubclass)hbmObject).Items);
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmSubclass)hbmObject).join);
                }
                else if (hbmObject is HbmJoin)
                {
                    applyUniqueConstraints(uniqueConstraintMap, ((HbmJoin)hbmObject).Items);
                }
            }
        }

        internal static HbmColumn[] getJoinColumns(JoinColumn[] joinColumns, Type targetType, ref String propertyRef)
        {
            if (joinColumns == null || joinColumns.Length == 0 || (joinColumns.Length == 1 && String.IsNullOrEmpty(joinColumns[0].Name)))
            {
                return null;
            }
            HbmColumn[] returnArr = new HbmColumn[joinColumns.Length];
            for (int i = 0; i < joinColumns.Length; i++)
            {
                JoinColumn joinCol = joinColumns[i];
                returnArr[i] = new HbmColumn(joinCol);
                if (!String.IsNullOrEmpty(joinCol.ReferencedColumnName) && String.IsNullOrEmpty(propertyRef))
                {
                    handlePropertyRef(joinCol.ReferencedColumnName, targetType, ref propertyRef);
                }
            }
            return returnArr;
        }

        internal static void handlePropertyRef(String ReferencedColumnName, Type targetType, ref String propertyRef)
        {
            if (String.IsNullOrEmpty(ReferencedColumnName))
            {
                return;
            }

            PropertyHelper p = MappingUtils.findPropertyByColumn(targetType, ReferencedColumnName);
            if (p == null)
            {
                throw new PersistenceException("ReferencedColumnName attribute <" + ReferencedColumnName + "> was not found in class <" + targetType.Name + ">.");
            }
            //String pk;
            //MappingUtils.getPKType(targetType, out pk);
            //if (p.Name.Equals(pk))
            //{
            //    throw new PersistenceException("ReferencedColumnName attribute <" + ReferencedColumnName + "> can't point to the entity ID property");
            //}
            propertyRef = p.Name;
        }

        internal static Type findMapKeyClass(PropertyHelper propertyInfo, Type mapValueClass, out String outMapKeyName)
        {
            String mapKeyName = null;
            bool foundGeneric = false;
            Type retKeyClass;
            MapKeyClass mapKeyClass = propertyInfo.getSingleAttribute<MapKeyClass>();
            if (mapKeyClass != null)
            {
                retKeyClass = mapKeyClass.Value;
                if (retKeyClass == null)
                {
                    throw new PersistenceException("The required attribute 'Value' on 'MapKeyClass' is empty");
                }
            }
            else
            {
                MapKey mapKey = propertyInfo.getSingleAttribute<MapKey>();
                if (mapKey != null && !String.IsNullOrEmpty(mapKey.Name))
                {
                    mapKeyName = mapKey.Name;
                    retKeyClass = MappingUtils.findPropertyType(mapKeyName, mapValueClass);
                }
                else if (MappingUtils.isEntity(mapValueClass))// find PK type
                {
                    retKeyClass = MappingUtils.getPKType(mapValueClass, out mapKeyName);
                    if (retKeyClass == null)
                    {
                        throw new PersistenceException("PK not found on " + mapValueClass.FullName + ". A map key can't be defined as default if a PK doesn't exist on the target type.");
                    }
                }
                else if (propertyInfo.PropertyType.IsGenericType)
                {
                    foundGeneric = true;
                    retKeyClass = propertyInfo.PropertyType.GetGenericArguments()[0];
                }
                else
                {
                    throw new PersistenceException("Map key not found for <" + propertyInfo.Name + ">.");
                }
            }

            if (!foundGeneric && propertyInfo.PropertyType.IsGenericType)
            {
                Type genericType = propertyInfo.PropertyType.GetGenericArguments()[0];
                if (!genericType.Equals(retKeyClass))
                {
                    throw new PersistenceException("Problem with Dictionary: key type in generics <" + genericType.Name
                        + "> is different from key type as decalred by NPA annotations <" + retKeyClass + ">.");
                }
            }

            outMapKeyName = mapKeyName;
            return retKeyClass;
        }

        internal static Column findSingleColumn(String propertyName, Type type)
        {
            PropertyInfo propertyInfo = type.GetProperty(propertyName);
            if (propertyInfo == null)
            {
                throw new ArgumentException("Could not map property: " + propertyName + ". Are you missing a relationship annotation?");
            }
            PropertyHelper propInfo = new PropertyHelper(propertyInfo);
            Column col = propInfo.getSingleAttribute<Column>();
            if (col != null)
            {
                if (string.IsNullOrEmpty(col.Name))
                {
                    col.Name = propInfo.Name;
                }
                return col;
            }
            return new Column
            {
                Name = propInfo.Name
            };
        }
    }
}
