﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    public class PropertyHelper
    {
        private PropertyInfo propertyInfo;
        private PropertyHelper parentEmbeddingProperty;
        private String keyValueMark;
        private bool onMappedSuperclass = false;
        public PropertyHelper(PropertyInfo propInfo, PropertyHelper parentEmbeddingProperty, String keyValueMark)
        {
            if (parentEmbeddingProperty == null)
            {
                throw new NullReferenceException();
            }
            propertyInfo = propInfo;
            this.parentEmbeddingProperty = parentEmbeddingProperty;
            this.keyValueMark = keyValueMark;
        }
        public PropertyHelper(PropertyInfo propInfo, Boolean onMappedSuperclass)
        {
            propertyInfo = propInfo;
            this.onMappedSuperclass = onMappedSuperclass;
        }

        public PropertyHelper(PropertyInfo propInfo):this(propInfo, false)
        {
        }

        public bool isOnMappedSuperclass()
        {
            return onMappedSuperclass;
        }

        public T getSingleAttribute<T>() where T : System.Attribute
        {
            T[] attArr = (T[])propertyInfo.GetCustomAttributes(typeof(T), false);
            return attArr != null && attArr.Length > 0 ? attArr[0] : null;
        }

        public T[] getAttributeArr<T>() where T : System.Attribute
        {
            return (T[])propertyInfo.GetCustomAttributes(typeof(T), false);
        }

        public Object[] GetCustomAttributes(bool inherit)
        {
            return propertyInfo.GetCustomAttributes(inherit);
        }

        public string Name
        {
            get { return propertyInfo.Name; } 
        }

        public Type PropertyType { get { return propertyInfo.PropertyType; } }

        public Type ReflectedType { 
            get 
            {
                if (parentEmbeddingProperty != null)
                {
                    return parentEmbeddingProperty.ReflectedType;
                }
                return propertyInfo.ReflectedType;
            } 
        }

        public Type DeclaringType
        {
            get
            {
                return propertyInfo.DeclaringType;
            }
        }

        public PropertyHelper getEmbeddingProperty(out String embeddingPath)
        {
            if (parentEmbeddingProperty == null)
            {
                embeddingPath = "";
                return this;
            }
            
            PropertyHelper ret = parentEmbeddingProperty.getEmbeddingProperty(out embeddingPath);
            if (!embeddingPath.Equals(""))
            {
                embeddingPath = embeddingPath + ".";
            } 
            embeddingPath = embeddingPath + propertyInfo.Name;
            return ret;
        }

        public String getPath()
        {
            if (parentEmbeddingProperty == null)
            {
                return propertyInfo.Name;
            }

            String path = parentEmbeddingProperty.getPath();
            path = path + ".";
            if (!String.IsNullOrEmpty(keyValueMark))
            {
                path = path + keyValueMark + ".";
            }
            path = path + propertyInfo.Name;
            return path;
        }

        public bool isEmbedded()
        {
            Embedded embedded = getSingleAttribute<Embedded>();
            if (embedded != null)
            {
                return true;
            }

            ElementCollection col = getSingleAttribute<ElementCollection>();
            if (col != null)
            {
                return true;
            }
            return false;
        }

        public Column findColumn(bool create)
        {
            Column col = getSingleAttribute<Column>();
            if (col != null)
            {
                if (string.IsNullOrEmpty(col.Name))
                {
                    col.Name = propertyInfo.Name;
                }
                return col;
            }
            if (create)
            {
                return new Column
                {
                    Name = propertyInfo.Name
                };
            }
            return null;
        }
    }
}
