﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NPersistence.NHibernate.annotationSupport.mapping;
using NPersistence.Ext;
using NHibernate.Dialect;
using NPersistence.NHibernate.annotationSupport;
using NHibernate.Type;

namespace NPersistence.NHibernate.src.annotationSupport.mapping
{
    class PropertyUtils
    {
        internal static void handleColumn(PropertyHelper propertyInfo, ref Object[] items)
        {
            Column col = propertyInfo.findColumn(true);
            items = new HbmColumn[] { new HbmColumn(col, propertyInfo) };
        }

        internal static void handleIndex(PropertyHelper propertyInfo, HbmColumn hbmCol)
        {
            Index ix = propertyInfo.getSingleAttribute<Index>();
            if (ix != null)
            {
                if (string.IsNullOrEmpty(ix.Name))
                {
                    ix.Name = MappingUtils.getNextIndexCounter() + "_ix";
                }
                hbmCol.index = String.IsNullOrEmpty(hbmCol.index) ? ix.Name : hbmCol.index + "," + ix.Name;
            }
        }

        internal static void handleUniqueConstrains(PropertyHelper propertyInfo, HbmColumn hbmCol)
        {
            Unique[] uniqueArr = propertyInfo.getAttributeArr<Unique>();
            if (uniqueArr != null)
            {
                foreach (Unique uniqueConstraint in uniqueArr)
                {
                    if (string.IsNullOrEmpty(uniqueConstraint.Name))
                    {
                        hbmCol.unique = true;
                        hbmCol.uniqueSpecified = true;
                    }
                    else
                    {
                        hbmCol.uniquekey = String.IsNullOrEmpty(hbmCol.uniquekey) ? uniqueConstraint.Name : hbmCol.uniquekey + "," + uniqueConstraint.Name;
                    }
                }
            }
        }

        internal static void handleBasic(PropertyHelper propertyInfo, HbmColumn hbmCol, ref bool lazy)
        {
            Basic basicAtt = propertyInfo.getSingleAttribute<Basic>();
            if (basicAtt != null)
            {
                if (basicAtt.Fetch == FetchType.EAGER)
                {
                    lazy = false;
                }
                if (basicAtt.Optional == false)
                {
                    hbmCol.notnull = true;
                    hbmCol.notnullSpecified = true;
                }
            }
        }

        internal static String getEnumType(EnumType enumType, Type enumTyp)
        {
            String retType;
            if (enumType == EnumType.STRING)
            {
                Type enumStringType = typeof(EnumStringType);
                retType = enumStringType.FullName + "`1[[" + MappingUtils.getStringClassDef(enumTyp) + "]]," + enumStringType.Assembly.GetName().Name;
            }
            else
            {
                retType = MappingUtils.getStringClassDef(enumTyp);
            }
            return retType;
        }

        internal static System.Data.DbType getTemporalSqlType(TemporalType temporal)
        {
            System.Data.DbType dbType;
            if (temporal == TemporalType.DATE)
            {
                dbType = System.Data.DbType.Date;
            }
            else if (temporal == TemporalType.TIME)
            {
                dbType = System.Data.DbType.Time;
            }
            else
            {
                dbType = System.Data.DbType.DateTime;
            }
            return dbType;
        }

        internal static void handlePropertyType(PropertyHelper propertyInfo, Type type, HbmColumn hbmCol, ref String typeName, bool isMapKey)
        {
            TemporalType? temporalType = null;
            EnumType? enumType = null;

            if (isMapKey)
            {
                MapKeyTemporal temporal = propertyInfo.getSingleAttribute<MapKeyTemporal>();
                if (temporal != null)
                {
                    temporalType = temporal.Value;
                }
                else
                {
                    MapKeyEnumerated enumerated = propertyInfo.getSingleAttribute<MapKeyEnumerated>();
                    if (enumerated != null)
                    {
                        enumType = enumerated.Value;
                    }
                }
            }
            else
            {
                Temporal temporal = propertyInfo.getSingleAttribute<Temporal>();
                if (temporal != null)
                {
                    temporalType = temporal.Value;
                }
                else
                {
                    Enumerated enumerated = propertyInfo.getSingleAttribute<Enumerated>();
                    if (enumerated != null)
                    {
                        enumType = enumerated.Value;
                    }
                }
            }

            if (temporalType.HasValue)
            {
                typeName = MappingUtils.getStringClassDef(type);
                System.Data.DbType dbType = getTemporalSqlType(temporalType.Value);
                Dialect dialect = AnnotaionConfig.getInstance(null).getDialect();
                hbmCol.sqltype = dialect.GetLongestTypeName(dbType);
            }
            else if (enumType.HasValue)
            {
                typeName = getEnumType(enumType.Value, type);
            }
            else
            {
                Lob lob = propertyInfo.getSingleAttribute<Lob>();
                if (lob != null)
                {
                    if (typeof(String).IsAssignableFrom(type)
                    || typeof(string).IsAssignableFrom(type)
                    || typeof(char[]).IsAssignableFrom(type))
                    {
                        typeName = "StringClob";
                        Dialect dialect = AnnotaionConfig.getInstance(null).getDialect();
                        hbmCol.sqltype = dialect.GetLongestTypeName(System.Data.DbType.String);
                    }
                    else
                    {
                        typeName = "binary";
                    }
                }
                else
                {
                    typeName = MappingUtils.getStringClassDef(type);
                }
            }
        }

    }
}
