

using NPersistence;
using NHibernate;
using System;
namespace NPersistence.NHibernate.util
{
    public class LockModeTypeHelper
    {
        public static LockModeType getLockModeType(LockMode lockMode)
        {
            if (lockMode == LockMode.None)
            {
                return LockModeType.NONE;
            }
            else if (lockMode == LockMode.Read /* || lockMode == LockMode.OPTIMISTIC  */ )
            {
                return LockModeType.OPTIMISTIC;
            }
            else if (lockMode == /*LockMode.OPTIMISTIC_FORCE_INCREMENT || lockMode == */ LockMode.Write)
            {
                return LockModeType.OPTIMISTIC_FORCE_INCREMENT;
            }
            // Not supported for NPA-Nhiberante
            //else if ( lockMode == LockMode.PESSIMISTIC_READ ) {
            //	return LockModeType.PESSIMISTIC_READ;
            //}
            else if ( //lockMode == LockMode.PESSIMISTIC_WRITE ||
                     lockMode == LockMode.Upgrade
                    || lockMode == LockMode.UpgradeNoWait)
            {
                return LockModeType.PESSIMISTIC_WRITE;
            }
            else if ( //lockMode == LockMode.PESSIMISTIC_FORCE_INCREMENT ||
                     lockMode == LockMode.Force)
            {
                return LockModeType.PESSIMISTIC_FORCE_INCREMENT;
            }
            throw new AssertionFailure("unhandled lock mode " + lockMode);
        }


        public static LockMode getLockMode(LockModeType lockMode)
        {
            switch (lockMode)
            {
                case LockModeType.READ:
                case LockModeType.OPTIMISTIC:
                    {
                        return LockMode.Read;
                    }
                case LockModeType.OPTIMISTIC_FORCE_INCREMENT:
                case LockModeType.WRITE:
                    {
                        return LockMode.Write;
                    }
                // Not supported for NPA-Nhiberante
                /*case LockModeType.PESSIMISTIC_REA: 
                    {
                        return LockMode.PESSIMISTIC_READ;
                    }*/
                case LockModeType.PESSIMISTIC_WRITE:
                    {
                        return LockMode.Upgrade;
                    }
                case LockModeType.PESSIMISTIC_FORCE_INCREMENT:
                    {
                        return LockMode.Force;
                    }
                case LockModeType.NONE:
                    {
                        return LockMode.None;
                    }
                default:
                    {
                        throw new AssertionFailure("Unknown LockModeType: " + lockMode);
                    }
            }
        }

        public static LockMode interpretLockMode(Object value)
        {
            if (value == null)
            {
                return LockMode.None;
            }
            if (value is LockMode)
            {
                return (LockMode)value;
            }
            else if (value is LockModeType)
            {
                return getLockMode((LockModeType)value);
            }
            else if (value is string)
            {
                // first try LockMode name
                LockMode lockMode = parseLockMode((string)value);
                if (lockMode == null)
                {
                    try
                    {
                        lockMode = getLockMode(parseLockModeType((String)value).Value);
                    }
                    catch (Exception ignore)
                    {
                    }
                }
                if (lockMode != null)
                {
                    return lockMode;
                }
            }

            throw new ArgumentException("Unknown lock mode source : " + value);
        }

        private static LockMode parseLockMode(string lockMode)
        {
            lockMode = lockMode.ToLower();
            if (lockMode.Equals("force"))
            {
                return LockMode.Force;
            }
            else if (lockMode.Equals("none"))
            {
                return LockMode.None;
            }
            else if (lockMode.Equals("read"))
            {
                return LockMode.Read;
            }
            else if (lockMode.Equals("upgrade"))
            {
                return LockMode.Upgrade;
            }
            else if (lockMode.Equals("upgradenowait"))
            {
                return LockMode.UpgradeNoWait;
            }
            return null;

        }

        private static LockModeType? parseLockModeType(string lockModeType)
        {
            lockModeType = lockModeType.ToLower();
            if (lockModeType.Equals("none"))
            {
                return LockModeType.NONE;
            }
            else if (lockModeType.Equals("optimistic"))
            {
                return LockModeType.OPTIMISTIC;
            }
            else if (lockModeType.Equals("optimistic_force_increment"))
            {
                return LockModeType.OPTIMISTIC_FORCE_INCREMENT;
            }
            else if (lockModeType.Equals("pessimistic_force_increment"))
            {
                return LockModeType.PESSIMISTIC_FORCE_INCREMENT;
            }
            else if (lockModeType.Equals("pessimistic_write"))
            {
                return LockModeType.PESSIMISTIC_WRITE;
            }
            else if (lockModeType.Equals("read"))
            {
                return LockModeType.READ;
            }
            else if (lockModeType.Equals("write"))
            {
                return LockModeType.WRITE;
            }
            return null;
        }
    }
}